@@ -35,4 +35,5 @@ ztfy_thesaurus_css = Resource(library, 'css/ztfy.thesaurus.css')
 ztfy_thesaurus = Resource(library, 'js/ztfy.thesaurus.js',
                           minified='js/ztfy.thesaurus.min.js',
                           depends=[ztfy_blog_back, jquery_alerts, jquery_jsonrpc,
-                                   ztfy_thesaurus_css])
+                                   ztfy_thesaurus_css],
+                          bottom=True)
