@@ -17,8 +17,7 @@
 # import standard packages
 
 # import Zope3 interfaces
-from z3c.form.interfaces import DISPLAY_MODE
-from zope.copypastemove.interfaces import IObjectMover
+from z3c.form.interfaces import DISPLAY_MODE, IErrorViewSnippet
 
 # import local interfaces
 from ztfy.thesaurus.interfaces.term import IThesaurusTermInfo
@@ -26,6 +25,8 @@ from ztfy.thesaurus.interfaces.thesaurus import IThesaurus
 
 # import Zope3 packages
 from z3c.form import field
+from zope.component import getMultiAdapter
+from zope.interface import Invalid
 from zope.security.proxy import removeSecurityProxy
 from zope.traversing import api as traversing_api
 
@@ -56,10 +57,21 @@ class ThesaurusTermAddForm(DialogAddForm):
     def title(self):
         return getParent(self.context, IThesaurus).title
 
+    def extractData(self, setErrors=True):
+        data, errors = super(ThesaurusTermAddForm, self).extractData(setErrors=setErrors)
+        if data.get('label') in IThesaurus(self.context).terms:
+            error = Invalid(_("New label already in use"))
+            widget = self.widgets['label']
+            view = getMultiAdapter((error, self.request, widget, widget.field, self, self.context),
+                                   IErrorViewSnippet)
+            view.update()
+            errors += (view,)
+            if setErrors:
+                widget.error = view
+                self.widgets.errors = errors
+        return data, errors
+
     def create(self, data):
-        thesaurus = IThesaurus(self.context)
-        if data.get('label') in thesaurus.terms:
-            raise ValueError, _("The given label is already used in this thesaurus !")
         result = ThesaurusTerm(data.get('label'))
         setRequestData('thesaurus.new_term', result, self.request)
         return result
@@ -118,15 +130,34 @@ class ThesaurusTermEditForm(DialogEditForm):
         self.widgets['extracts'].mode = DISPLAY_MODE
         self.widgets['created'].mode = DISPLAY_MODE
 
+    def extractData(self, setErrors=True):
+        data, errors = super(ThesaurusTermEditForm, self).extractData(setErrors=setErrors)
+        label = data.get('label')
+        if (label != self.context.label) and (data.get('label') in traversing_api.getParent(self.context)):
+            error = Invalid(_("New label already in use"))
+            widget = self.widgets['label']
+            view = getMultiAdapter((error, self.request, widget, widget.field, self, self.context),
+                                   IErrorViewSnippet)
+            view.update()
+            if setErrors:
+                widget.error = view
+                self.widgets.errors = errors
+            errors += (view,)
+        return data, errors
+
     def applyChanges(self, data):
         term = self.getContent()
         thesaurus = getParent(term, IThesaurus)
+        old_label = term.label
         old_generic = term.generic
         old_usage = term.usage
         changes = super(ThesaurusTermEditForm, self).applyChanges(data)
         # Move term if label changed
+        # Don't use IObjectMover interface to avoid cataloging problems...
         if 'label' in changes.get(IThesaurusTermInfo, []):
-            IObjectMover(self.context).moveTo(traversing_api.getParent(term), term.label)
+            parent = traversing_api.getParent(term)
+            del parent[old_label]
+            parent[term.label] = term
         # Check terms and thesaurus top terms if generic or usage changed
         self._v_generic_changed = old_generic != term.generic
         self._v_usage_changed = old_usage != term.usage
@@ -188,7 +219,8 @@ class ThesaurusTermEditForm(DialogEditForm):
         if self._v_generic_changed:
             status = u'RELOAD'
         else:
-            if 'label' in changes.get(IThesaurusTermInfo, []):
+            term_changes = changes.get(IThesaurusTermInfo, [])
+            if ('label' in term_changes) or ('status' in term_changes):
                 term = self.getContent()
                 generic = term.generic
                 if generic is None:
@@ -196,7 +228,8 @@ class ThesaurusTermEditForm(DialogEditForm):
                 else:
                     status = u'CALLBACK'
                     callback = '$.ZTFY.thesaurus.tree.reloadTerm'
-                    options = { 'source': generic.label.replace("'", "&#039;") }
+                    options = { 'source': generic.label.replace("'", "&#039;"),
+                                'status': term.status }
             else:
                 status = changes and u'OK' or u'NONE'
         return writer.write({ 'output': status,
