@@ -27,18 +27,20 @@ from ztfy.thesaurus.interfaces.thesaurus import IThesaurus, IThesaurusExtracts
 from ztfy.thesaurus.interfaces.tree import ITree, INode
 
 # import Zope3 packages
-from z3c.formjs import ajax
+from z3c.form import field, button
+from z3c.formjs import ajax, jsaction
 from zope.component import getUtility
 from zope.i18n import translate
-from zope.interface import implements
+from zope.interface import implements, Interface
 from zope.traversing.api import getName
 from zope.traversing.browser.absoluteurl import absoluteURL
 
 # import local packages
+from ztfy.i18n.property import translated
 from ztfy.jqueryui import jquery_datetime, jquery_multiselect
-from ztfy.skin.form import DialogDisplayForm
+from ztfy.skin.form import DialogDisplayForm, DialogEditForm
 from ztfy.skin.menu import MenuItem
-from ztfy.skin.page import BaseBackView, TemplateBasedPage
+from ztfy.skin.page import BaseBackView, BaseTemplateBasedPage, TemplateBasedPage
 from ztfy.thesaurus.browser import ztfy_thesaurus
 from ztfy.utils.property import cached_property
 from ztfy.utils.traversing import getParent
@@ -197,3 +199,59 @@ class ThesaurusExtractTermsTreeView(DialogDisplayForm):
         writer = getUtility(IJSONWriter)
         return writer.write({ 'term': term.label,
                               'nodes': sorted(result, key=lambda x: x['label']) })
+
+
+#
+# Thesaurus terms selector
+#
+
+class ISelectorDialogFormButtons(Interface):
+    """Terms selector dialog form buttons"""
+
+    select = jsaction.JSButton(title=_("Select terms"))
+    cancel = jsaction.JSButton(title=_("Cancel"))
+
+
+class ThesaurusTermsSelectorView(DialogEditForm):
+    """Thesaurus terms selector tree view"""
+
+    fields = field.Fields(Interface)
+    buttons = button.Buttons(ISelectorDialogFormButtons)
+
+    @property
+    def title(self):
+        return self.context.title
+
+    @translated
+    def legend(self):
+        return _("Thesaurus terms selection")
+
+    @cached_property
+    def thesaurus(self):
+        return getParent(self.context, IThesaurus)
+
+    @property
+    def top_terms(self):
+        extract = self.request.form.get('extract')
+        return sorted([term for term in self.context.top_terms
+                                     if (not extract) or (extract in term.extracts) ],
+                      key=lambda x: x.label)
+
+    @jsaction.handler(buttons['select'])
+    def add_handler(self, event, selector):
+        return '$.ZTFY.thesaurus.tree.openSelectorAction(this.form);'
+
+    @jsaction.handler(buttons['cancel'])
+    def cancel_handler(self, event, selector):
+        return '$.ZTFY.dialog.close();'
+
+
+class ThesaurusTermsSelectorNodeView(BaseTemplateBasedPage):
+    """Thesaurus terms selector node view"""
+
+    @property
+    def sub_terms(self):
+        extract = self.request.form.get('extract')
+        return sorted([term for term in self.context.specifics
+                                     if (not extract) or (extract in term.extracts) ],
+                      key=lambda x: x.label)
