@@ -41,6 +41,7 @@ from ztfy.jqueryui import jquery_multiselect
 from ztfy.thesaurus.browser import ztfy_thesaurus
 from ztfy.utils.traversing import getParent
 from ztfy.thesaurus.interfaces.term import IThesaurusTerm
+from zope.traversing.browser.absoluteurl import absoluteURL
 
 
 SYNONYM = re.compile('(.*)\ \[\ .*\ \]')
@@ -143,8 +144,9 @@ class ThesaurusTermsListWidget(TextWidget):
 
     implementsOnly(IThesaurusTermsListFieldWidget)
 
-    thesaurus_name = FieldProperty(IThesaurusTermFieldWidget['thesaurus_name'])
-    extract_name = FieldProperty(IThesaurusTermFieldWidget['extract_name'])
+    thesaurus_name = FieldProperty(IThesaurusTermsListFieldWidget['thesaurus_name'])
+    extract_name = FieldProperty(IThesaurusTermsListFieldWidget['extract_name'])
+    display_terms_selector = FieldProperty(IThesaurusTermsListFieldWidget['display_terms_selector'])
 
     size = 10
     multiple = True
@@ -158,6 +160,11 @@ class ThesaurusTermsListWidget(TextWidget):
         self.terms = []
         return self.terms
 
+    def getThesaurusURL(self):
+        thesaurus = queryUtility(IThesaurus, self.thesaurus_name)
+        if thesaurus is not None:
+            return absoluteURL(thesaurus, self.request)
+
     def render(self):
         jquery_multiselect.need()
         ztfy_thesaurus.need()
