@@ -22,7 +22,7 @@ from zope.container.interfaces import IContainer
 # import local interfaces
 
 # import Zope3 packages
-from zope.interface import Interface, Attribute
+from zope.interface import Interface, Attribute, Invalid, invariant
 from zope.schema import Text, TextLine, Bool, Choice, Int, Datetime, Set
 from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm
 
@@ -124,7 +124,8 @@ class IThesaurusTermInfo(Interface):
                        required=False)
 
     modified = Datetime(title=_("term-modified-title", "Modification date"),
-                        required=False)
+                        required=False,
+                        readonly=True)
 
     def getParents(self):
         """Get list of term's parents"""
@@ -138,6 +139,14 @@ class IThesaurusTermInfo(Interface):
     def queryExtensions(self):
         """Get list of extension utilities"""
 
+    @invariant
+    def checkSynonym(self):
+        if self.usage is not None:
+            if self.generic is not None:
+                raise Invalid(_("A term can't be a synonym and attached to a generic term"))
+            if self.used_for:
+                raise Invalid(_("A term used as synonym can't have it's own synonyms (all synonyms should be attached to the descriptor)"))
+
 
 class IThesaurusTermWriter(Interface):
     """Thesaurus term writer interface"""
